#!/bin/sh

#echo "p0=$0, p1=$1, p2=$2, p3=$3" >> /tmp/tmp.PatchEXE

G_BE_OS_UPDATE="0"
if [ "$1" == "apply" ] && [ "$2" != "" ]; then
	G_BE_OS_UPDATE="1"
elif [ "$1" == "untar" ] && [ "`echo $2 | cut -d"/" -f 4`" != "AdminUI" ]; then
	G_BE_OS_UPDATE="1"	
fi

if [ "$G_BE_OS_UPDATE" == "1" ]; then
	ROOT=/etc/iscan/os_patch
	INSTALL_ROOT=/etc/iscan/os_patch
	PATCH_SUCCESS_FILE=/tmp/patch_os.success
	#protect from apply run twice at the same time
	PATCH_LOCK_FILE=/tmp/lock_patch_os_apply
	
	#. localization strings for OS Update
	strPatchNumber="Patch-Number"
	strPatchTitle="Patch-Title"
	strPatchDescription="Patch-Description"
	strRestartUI="Restart-UI"
	strRestartWeb="Restart-Web"
	strRestartMail="Restart-Mail"
	strRestartSystem="Restart-System"
	strError="Error"
	strFinished="Finished"
	#. localization strings for OS Update end---------------------------	
else
	ROOT=/etc/iscan/patch
	INSTALL_ROOT=/etc/iscan
	PATCH_SUCCESS_FILE=/tmp/patch.success
	#protect from apply run twice at the same time
	PATCH_LOCK_FILE=/tmp/lock_patch_apply

	strPatchNumber="Patch-Number"
	strPatchTitle="Patch-Title"
	strPatchDescription="Patch-Description"
	strRestartUI="Restart-UI"
	strRestartWeb="Restart-Web"
	strRestartMail="Restart-Mail"
	strRestartSystem="Restart-System"
	strError="Error"
	strFinished="Finished"	
fi
export G_BE_OS_UPDATE

. ./ini_util.sh


_SAVED_PWD=
F_pushd() {
	_SAVED_PWD=`pwd`
}
F_popd() {
	 cd $_SAVED_PWD
}

if [ "$1" = "untar" ] ; then
	FILE1="$2"
elif [ "$1" = "apply" ] ; then
	printf ""	
else
	echo "Usage: $0 untar example_patch.tar  | apply"
	exit 1
fi


BACKUP_DIR=${ROOT}/backup

FILE2_ORIG=stargate_patch.tgz
FILE2=${EXTRACT_DIR}/${FILE2_ORIG}
FILE2_MD5=${EXTRACT_DIR}/MD5SUM.txt

APPLY_SCRIPT=stargate_patch_apply.sh
ROLLBACK_SCRIPT=stargate_patch_rollback.sh
THISPATCHINI=stargate_patch.ini
README=README.txt

PATCH_MGM_INI=${ROOT}/config/patch_management.ini

F_untar() {
	#need a absolute path to the tar
	_FILE=$1
	_DIR=$2
	F_pushd
	cd $_DIR
	echo "Decompressing ${_FILE} to ${_DIR}."
	/bin/tar zxvf ${_FILE}
	if [ $? != '0' ] ; then
		 echo "Unable to decompress ${_FILE}."
		F_popd
		return 1
	fi
	 find -type f 
	 echo "Decompression OK."
	F_popd
	return 0
}

F_file_exist() {
	_F=$1
	if [ ! -r "$_F" ] ; then
		 echo "${_F} not found."
		return 1
	fi
	return 0
}

F_dir_exist() {
	_D=$1
	if [ ! -d "$_D" ] ; then
		 echo "Dirrectory ${_D} not found."
		return 1
	fi
	return 0
}

F_untar_size() {
	# Calculate untar size. Get echo file size from verbose file list and add them together in Kilo Byte
	_FILE=$1
	_R=`tar ztvf ${_FILE} | awk ' { print "( " ; printf $3; print " / 1024 ) + " } '  | xargs`
	_R="$_R 0 "
	# echo $_R
	_UNTAR_SIZE=`expr $_R`
	echo $_UNTAR_SIZE
	return 0
}

F_check_disk_space() {
	SPACE_MIN="$2"
	DIR_TO_CHECK="$1"
echo "SPACE_MIN=$SPACE_MIN, DIR_TO_CHECK=$1"
#	DF_STR=`df -k ${DIR_TO_CHECK} | tail +2`
	DF_STR=`df -k ${DIR_TO_CHECK} | tail -n +2`
	I=0
	for AVAIL_SPACE in ${DF_STR}
	do
		if [ "$I" = "3" ] ; then
			if [ "${AVAIL_SPACE}" -ge "${SPACE_MIN}" ] ; then
				return 0
			else
				return 1
			fi
		fi
		I=`expr $I + 1`
	done
	return 1
}

F_checksum_ok() {
	_FF=$1
	# Use the md5 to check sum
	_TAR_MD5=`grep "${FILE2_ORIG}" "$_FF" | awk '{print $1 }'`

	_FILE_MD5=`md5sum "$FILE2_ORIG" | awk '{print $1}'`
	if [ "$_TAR_MD5" != "$_FILE_MD5" ] ; then
		 echo "The checksum was incorrect."
		return 1
	fi
	 echo "The checksum was correct."
	return 0
}

F_first_untar() {

	F_file_exist ${FILE1}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0001] The patch system cannot find the compressed patch file (titled ${FILE1}) that has been uploaded, and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0001] Cannot find ${FILE1}."
		return 1
	fi

	#clean up extract dir
	rm -rf ${EXTRACT_DIR}
	mkdir -p ${EXTRACT_DIR}
	
	chmod 777 ${EXTRACT_DIR}

	mv ${FILE1} ${EXTRACT_DIR}
	
	FFILE1=`basename ${FILE1}`
	cd ${EXTRACT_DIR}
	#untar level1
	F_untar ${EXTRACT_DIR}/${FFILE1} ${EXTRACT_DIR}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0002]   IWSS could not extract the specified file: ${FFILE1}. Please make sure it is a valid IWSS patch file and try again."
		#"[ERR-PAT-0002] Cannot decompress first ${FFILE1}."
		return 1
	fi


	#We require a specific tar and a md5 to exist.
	F_file_exist "$FILE2"
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0003]   The patch system cannot find the file (titled ${FILE2}) in the decompressed set of patch files, and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0003] Cannot find `basename ${FILE2}`."
		return 1
	fi

	F_file_exist "$FILE2_MD5"
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0004]   The patch system cannot find the required MD5SUM checksum file titled ${FILE2_MD5} in the decompressed set of patch files, and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0004] Cannot find `basename ${FILE2_MD5}`."
		return 1
	fi

	# make sure the md5 has the check summed file 
	grep `basename "$FILE2"` "$FILE2_MD5" > /dev/null 2>&1
	if [ $? != "0" ] ; then
		 echo "File $FILE2_MD5 should only have a checksum of ${FILE2}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0005]   The MD5SUM file in the patch titled ${FILE2_MD5} doesn't have a checksum line. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0005] File `basename $FILE2_MD5` should only have a checksum of `basename ${FILE2}`"
		return 1
	fi

	F_checksum_ok ${FILE2_MD5}
	if [ $? != "0" ] ; then
		 echo "The checksum in `basename ${FILE2_MD5}` is incorrect."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0006]   The patch MD5SUM check was unsuccessful, and the patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0006] Checksum in `basename ${FILE2_MD5}` is incorrect."
		return 1
	fi
	 rm ${FILE2_MD5} 
	#done checksum
}


F_second_untar() {
	F_dir_exist ${EXTRACT_DIR}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}"  "[ERR-PAT-0007]   The patch system cannot find the temporary extraction directory titled ${EXTRACT_DIR}. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0007] Cannot find extracted directory ${EXTRACT_DIR}."
		return 1
	fi


	UNTAR_SIZE=`F_untar_size ${FILE2} `
	#plus 5 MB
	UNTAR_SIZE=`expr $UNTAR_SIZE + 5000`
	#check backup dir size larger than untar size
	F_check_disk_space ${EXTRACT_DIR} ${UNTAR_SIZE}
	if [ "$?" != "0" ] ; then
		 echo "Backup dir ${BACKUP_DIR} available disk space is too small."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0008]   Storage space on the SSM is insufficient to extract the patch files. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0008] Storage space is not enough to extract."
		return 2
	fi

	#extract after disk space check!
	F_untar ${FILE2} ${EXTRACT_DIR} 
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0009]   A compressed file inside the patch (titled ${FILE2}) cannot be decompressed. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0009] Cannot decompress ${FILE2}."
		return 1
	fi

	#Sim-Ship begin Need to overwrite L10N file based on System locale
    PATCH_BUILD=`F_ini_get ${THISPATCHINI} PatchInfo Build`
    TMP_INI="/tmp/extract_patch/stargate_patch.ini"
    _PRODUCT_LANG=`cat /etc/iscan/intscan.ini |grep lang=|cut -d '=' -f 2`
    _ja_des=`F_ini_get $TMP_INI PatchInfo Description_JA`
    _cn_des=`F_ini_get $TMP_INI PatchInfo Description_CN`
    _current_dir=`pwd`;
    echo "$_current_dir";
    if [ -z "$_ja_des" ]; then
            echo "Old hotfix/patch install..."
    else
            if [ "$_PRODUCT_LANG" = "ja" ] ; then
                    echo "JA Version, Need to overwrite JA L10N files"
                    cd /tmp/extract_patch/files/
                    /bin/tar -xvf /tmp/extract_patch/files/ja_l10n.tar
                    F_ini_set $TMP_INI PatchInfo Description "$_ja_des"

            elif [ "$_PRODUCT_LANG" = "zh" ] ; then
                    echo "ZH Version, Need to overwrite ZH L10N files"
                    cd /tmp/extract_patch/files/
                     /bin/tar -xvf /tmp/extract_patch/files/zh_l10n.tar
                     F_ini_set $TMP_INI PatchInfo Description "$_cn_des"

            elif [ "$_PRODUCT_LANG" = "en" ] ; then
                    echo "EN Version, No need to overwrite L10N files"

            else
            F_internal_error_report "The lanugage version is incorrect"
                    return 1
            fi
     fi
     cd $_current_dir

	#make sure there is a patch number
	PATCH_NUMBER=`F_ini_get ${THISPATCHINI} PatchInfo Number`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0010]   The patch is missing a patch number entry (from the configuration file titled ${THISPATCHINI}). The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0010] Cannot retrieve patch number from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strPatchNumber}" "${PATCH_NUMBER}"

	#make sure there is a patch title
	PATCH_TITLE=`F_ini_get ${THISPATCHINI} PatchInfo Title`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0011]   The patch is missing a patch name (from the file titled ${THISPATCHINI}) and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0011] Cannot retrieve patch title from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strPatchTitle}" "${PATCH_TITLE}"
	
	#make sure there is a patch description
	PATCH_DESC=`F_ini_get ${THISPATCHINI} PatchInfo Description`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0012]   The patch is missing a patch description (from the file titled ${THISPATCHINI}) and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support.
		"
		#"[ERR-PAT-0012] Cannot retrieve patch description from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strPatchDescription}" "${PATCH_DESC}"
	
	#requires field
	PATCH_REQUIRES=`F_ini_get ${THISPATCHINI} PatchInfo Requires`
	if [ -n "$PATCH_REQUIRES" ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "Patch-Requires" "${PATCH_REQUIRES}"
	fi
	
	#make sure there's an apply script
	F_file_exist ${APPLY_SCRIPT}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0013]   The patch is missing a required patch apply scripti titled ${APPLY_SCRIPT}, and cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0013] Cannot apply this patch. The ${APPLY_SCRIPT} is missing."
		return 1
	fi

	#UI restart
	RESTARTUI=`F_ini_get ${THISPATCHINI} PatchInfo RestartUI`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0014]   The patch configuration file \"stargate_patch.ini\" is missing a required flag to indicate whether the UI should be restarted. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0014] Cannot retrieve RestartUi from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strRestartUI}" "$RESTARTUI"

	#Web restart
	RESTARTWEB=`F_ini_get ${THISPATCHINI} PatchInfo RestartWeb`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0015]   The patch configuration file \"stargate_patch.ini\" is missing a required flag to indicate whether the httpd and ftpd should be restarted. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0015] Cannot retrieve RestartWeb from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strRestartWeb}" "$RESTARTWEB"
	
	#Mail restart
	RESTARTMAIL=`F_ini_get ${THISPATCHINI} PatchInfo RestartMail`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0016]   The patch configuration file \"stargate_patch.ini\" is missing a required flag to indicate whether the imssd should be restarted. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0016] Cannot retrieve RestartMail from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strRestartMail}" "$RESTARTMAIL"

	#System restart
	RESTARTSYSTEM=`F_ini_get ${THISPATCHINI} PatchInfo RestartSystem`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0017]   The patch configuration file \"stargate_patch.ini\" is missing a required flag to indicate whether the system should be restarted. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0017] Cannot retrieve RestartSystem from ${THISPATCHINI}."
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strRestartSystem}" "$RESTARTSYSTEM"

	return 0
}

F_decompress() {
	F_first_untar
	if [ $? -ne 0 ] ; then
		exit 1
	fi
	F_second_untar
	if [ $? -ne 0 ] ; then
		exit 1
	fi
	return 0
}

F_call_upload_check() {
	F_pushd
	cd ${EXTRACT_DIR} 
	if [ $? -ne 0 ] ; then
		 echo "The extracted directory doesn't exist. Please invoke '$0 untar ...' first"
		return 1
	fi
	#mkdir a backup dir according to patch number
	PATCH_NUMBER=`F_ini_get ${THISPATCHINI} PatchInfo Number`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0018]   The patch system is missing a patch number entry that should be available in the file titled ${THISPATCHINI}. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0018] Cannot retrieve patch number from ${THISPATCHINI}."
		return 1
	fi

	#BACKUP_DIR_FOR_1_PATCH=${BACKUP_DIR}/${PATCH_NUMBER}
	#mkdir -p ${BACKUP_DIR_FOR_1_PATCH}/files
	#BACKUP_DIR_FOR_1_PATCH=`cd ${BACKUP_DIR_FOR_1_PATCH} && pwd && cd - > /dev/null 2>&1`

	F_file_exist ${APPLY_SCRIPT}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0019]   After the upload, the patch system could not find the apply script titled ${APPLY_SCRIPT}. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0019] Cannot use this patch. The ${APPLY_SCRIPT} is missing."
		return 1
	fi
	
	#call the patch apply script to do upload check	
	/bin/sh ./${APPLY_SCRIPT} "upload_check" ${INSTALL_ROOT} ${BACKUP_DIR_FOR_1_PATCH} 
	_R=$?
	if [ $_R -ne 0 ] ; then
		ERR_MSG=""
		case "$_R" in 
		$ERR_REQ_F) ERR_MSG="[ERR-PAT-0020]   The patch configuration file is invalid (it either wasn't included in the patch, or is missing entries). The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support." 
		
		#"[ERR-PAT-0020] Cannot apply this patch. The patch configuration file is invalid." ;;
		;;
		$ERR_DEP)   ERR_MSG="[ERR-PAT-0021]   A patch with the same patch number is already installed. No further action is required."
		#"[ERR-PAT-0021] Cannot apply this patch. This patch has already been applied." ;;
		;;
		$ERR_REQ_P) ERR_MSG="[ERR-PAT-0022]   The patch requires another patch to be installed first. Please contact Technical Support."
		#"[ERR-PAT-0022] Cannot apply this patch. This patch requires previous patches." ;;
		;;
		$ERR_BAK)   ERR_MSG="[ERR-PAT-0023]   The patch system cannot back up the system for rollback purposes. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0023] Cannot apply this patch. Unable to backup the files." ;;
		;;
		$ERR_APP)   ERR_MSG="[ERR-PAT-0024]   The patch system cannot continue applying the patch. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0024] Cannot apply this patch. The system is not modified." ;;
		;;
		$ERR_VER)   ERR_MSG="[ERR-PAT-0025]   The patch system cannot continue applying the patch because one or more required files are missing. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0025] Cannot apply this patch. The patch lacks files." ;;
		;;
		$ERR_PATCH_OS) ERR_MSG="[ERR-PAT-0027]   This patch is used for OS update and cannot be applied."
		#"[ERR-PAT-0034] This patch is used for OS update and cannot be applied." ;;
		;;
		$ERR_PATCH_APP) ERR_MSG="[ERR-PAT-0028]   This patch is used for Application Patch and cannot be applied."
		#"[ERR-PAT-0032] This patch is used for Application Patch and cannot be applied." ;;
		;;
		*)          ERR_MSG="[ERR-PAT-0026]   The patch system cannot continue. The apply script returns error code ${_R}. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0026] Cannot apply this patch. ($_R)";;
		;;
		esac
	
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "$ERR_MSG"
		return 1
	fi
	F_popd
}

F_call_apply_now() {
	cd ${EXTRACT_DIR} 
	if [ $? -ne 0 ] ; then
		 echo "The extracted directory doesn't exist. Please invoke '$0 untar ...' first"
		return 1
	fi
	#mkdir a backup dir according to patch number
	PATCH_NUMBER=`F_ini_get ${THISPATCHINI} PatchInfo Number`
	if [ $? -ne 0 ] ; then
		echo "Cannot access ${THISPATCHINI}."
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0027]   The patch is missing a patch number entry (from the file titled ${THISPATCHINI}). The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0027] Cannot retrieve patch number from ${THISPATCHINI}."
		return 1
	fi

	if [ "$G_BE_OS_UPDATE" != "1" ]; then
		BACKUP_DIR_FOR_1_PATCH=${BACKUP_DIR}/${PATCH_NUMBER}
		mkdir -p ${BACKUP_DIR_FOR_1_PATCH}/files
	fi
	BACKUP_DIR_FOR_1_PATCH=`cd ${BACKUP_DIR_FOR_1_PATCH} && pwd && cd - > /dev/null 2>&1`
	F_file_exist ${APPLY_SCRIPT}
	if [ $? -ne 0 ] ; then
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "[ERR-PAT-0028]   The patch system cannot find the apply script titled ${APPLY_SCRIPT}, and the patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		#"[ERR-PAT-0028] Cannot use this patch. The ${APPLY_SCRIPT} is missing."
		return 1
	fi
	#call the patch backup apply script
	/bin/sh ./${APPLY_SCRIPT} "apply_now" ${INSTALL_ROOT} ${BACKUP_DIR_FOR_1_PATCH} 
	_R=$?
	if [ $_R -ne 0 ] ; then
		ERR_MSG=""
		
		case "$_R" in 
		$ERR_REQ_F) ERR_MSG="[ERR-PAT-0029]   The patch configuration file is invalid (it either wasn't included in the patch, or is missing entries). The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support." 
		;;
		$ERR_DEP)   ERR_MSG="[ERR-PAT-0030]   A patch with the same patch number is already installed. No further action is required."
		;;
		$ERR_REQ_P) ERR_MSG="[ERR-PAT-0031]   The patch requires another patch to be installed first. Please contact Technical Support."
		;;
		$ERR_BAK)   ERR_MSG="[ERR-PAT-0032]   The patch system cannot back up the system for rollback purposes. The patch cannot continue. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		;;
		$ERR_APP)   ERR_MSG="[ERR-PAT-0033]   The patch system cannot continue applying the patch. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		;;
		$ERR_VER)   ERR_MSG="[ERR-PAT-0034]   The patch system cannot continue applying the patch because one or more required files are missing. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		;;
		*)          ERR_MSG="[ERR-PAT-0035]   The patch system cannot continue. The apply script returns error code ${_R}. This message is for diagnostic purposes only. Customers - please contact Technical Support."
		;;
		esac
	
		F_internal_error_report "${PATCH_MGM_INI}" "${strError}" "$ERR_MSG"
		return 1
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "${strFinished}" "The patch application is finished."
	echo "You have successfully installed Patch ${PATCH_NUMBER}." > $PATCH_SUCCESS_FILE
	chown iscan:iscan ${PATCH_SUCCESS_FILE}
	rm -rf ${EXTRACT_DIR}

}
if [ "$1" = "untar" ] ; then
	F_log_init
	F_internal_error_report "${PATCH_MGM_INI}" "Ready-to-Apply" "Ready to apply patch."
	F_decompress
	F_call_upload_check
	_chk_result=$?
	rm -f ${PATCH_LOCK_FILE}
	if [ $_chk_result -ne 0 ]; then
		exit 1;
	fi	
else  
	# assume error check is done previously
	if [ -f "$PATCH_LOCK_FILE" ] ; then
		exit 0
	else
		touch $PATCH_LOCK_FILE
	fi
	F_internal_error_report "${PATCH_MGM_INI}" "Apply" "Begin to apply patch."
	F_call_apply_now
fi

